@@ -53,11 +53,12 @@
 #define SM_IO_WHAT_MODE		1
 #define SM_IO_WHAT_VECTORS	2
 #define SM_IO_WHAT_FD		3
-#define SM_IO_WHAT_TYPE		4
+/* was WHAT_TYPE		4 unused */
 #define SM_IO_WHAT_ISTYPE	5
 #define SM_IO_IS_READABLE	6
 #define SM_IO_WHAT_TIMEOUT	7
 #define SM_IO_WHAT_SIZE		8
+#define SM_IO_WHAT_BUF		9
 
 /* info flags (exposed) */
 #define SM_IO_FTYPE_CREATE	1
@@ -153,7 +154,9 @@ extern const char	SmFileMagic[];
 extern SM_FILE_T	SmFtStdio_def;
 extern SM_FILE_T	SmFtStdiofd_def;
 extern SM_FILE_T	SmFtString_def;
+#if 0
 extern SM_FILE_T	SmFtSyslog_def;
+#endif
 extern SM_FILE_T	SmFtRealStdio_def;
 
 #define SMIOIN_FILENO		0
@@ -174,7 +177,9 @@ extern SM_FILE_T	SmFtRealStdio_def;
 #define SmFtStdio	(&SmFtStdio_def)
 #define SmFtStdiofd	(&SmFtStdiofd_def)
 #define SmFtString	(&SmFtString_def)
+#if 0
 #define SmFtSyslog	(&SmFtSyslog_def)
+#endif
 #define SmFtRealStdio	(&SmFtRealStdio_def)
 
 #ifdef __STDC__
@@ -342,7 +347,7 @@ __END_DECLS
 __BEGIN_DECLS
 int	sm_rget __P((SM_FILE_T *, int));
 int	sm_vfscanf __P((SM_FILE_T *, int SM_NONVOLATILE, const char *,
-			va_list SM_NONVOLATILE));
+			va_list));
 int	sm_wbuf __P((SM_FILE_T *, int, int));
 __END_DECLS
 
@@ -383,7 +388,7 @@ __END_DECLS
 # ifndef _POSIX_SOURCE
 #  define sm_io_getc(fp, t)	sm_getc(fp, t)
 #  define sm_io_putc(fp, t, x)	sm_putc(fp, t, x)
-# endif /* _POSIX_SOURCE */
+# endif
 #endif /* lint */
 
 #endif /* SM_IO_H */
