@@ -846,7 +846,7 @@ store(from, inbody)
 			if (line[1] == '\n' ||
 			    (line[1] == '\r' && line[2] == '\n'))
 				goto lmtpdot;
-			memcpy(line, line + 1, line_len);
+			memmove(line, line + 1, line_len);
 			line_len--;
 		}
 
