@@ -348,7 +348,9 @@ mci_clear(mci)
 #if _FFR_SE_TA_ID
 	mci->mci_c_se = NULL;
 #endif
-
+#if _FFR_MTA_STS
+	/* MCIF_HASSTS is implicitly cleared: */
+#endif
 	/* which flags to preserve? */
 	mci->mci_flags &= MCIF_CACHED;
 	mactabclear(&mci->mci_macro);
@@ -654,6 +656,9 @@ static struct mcifbits	MciFlags[] =
 	{ MCIF_NOTSTICKY,	"NOTSTICKY"	},
 #if USE_EAI
 	{ MCIF_EAI,		"EAI"		},
+#endif
+#if _FFR_MTA_STS
+	{ MCIF_HASSTS,		"HASSTS"	},
 #endif
 	{ 0,			NULL		}
 };
