@@ -2495,6 +2495,9 @@ static struct qflags	AddressFlags[] =
 	{ "QQUEUED",		QQUEUED		},
 	{ "QINTREPLY",		QINTREPLY	},
 	{ "QMXSECURE",		QMXSECURE	},
+#if _FFR_MTA_STS
+	{ "QHASSTS",		QHASSTS		},
+#endif
 	{ "QTHISPASS",		QTHISPASS	},
 	{ "QRCPTOK",		QRCPTOK		},
 	{ NULL,			0		}
