@@ -436,7 +436,7 @@ savemail(e, sendbody)
 
 			if (!putfromline(&mcibuf, e) ||
 			    !(*e->e_puthdr)(&mcibuf, e->e_header, e,
-					M87F_OUTER) ||
+					M87F_OUTER, NULL) ||
 			    !(*e->e_putbody)(&mcibuf, e, NULL) ||
 			    !putline("\n", &mcibuf) ||
 			    sm_io_flush(fp, SM_TIME_DEFAULT) == SM_IO_EOF ||
@@ -1482,7 +1482,7 @@ errbody(mci, e, separator)
 			goto writeerr;
 		save_errno = errno;
 		if (!putheader(mci, e->e_parent->e_header, e->e_parent,
-				M87F_OUTER))
+				M87F_OUTER, NULL))
 			goto writeerr;
 		errno = save_errno;
 		if (sendbody)
@@ -1641,6 +1641,7 @@ xtextify(t, taboo)
 	{
 		/* since nbogus is ssize_t and wrapped, 2 * size_t would wrap */
 		syserr("!xtextify string too long");
+		/* NOTREACHED */
 	}
 	if (nbogus == 0)
 		return t;
@@ -1888,7 +1889,7 @@ pruneroute(addr)
 	while (start != NULL)
 	{
 		if (getmxrr(hostbuf, mxhosts, NULL, TRYFALLBACK, &rcode, NULL,
-			    -1) > 0)
+			    -1, NULL) > 0)
 		{
 			(void) sm_strlcpy(addr + 1, start + 1,
 					  strlen(addr) - 1);
