@@ -2431,7 +2431,10 @@ cleanstrcpy(t, f, l)
 	(void) denlstring(f, true, true);
 
 	if (l <= 0)
-		syserr("!cleanstrcpy: length == 0");
+	{
+		syserr("!cleanstrcpy: l=%d, status=invalid", l);
+		/* NOTREACHED */
+	}
 
 	l--;
 	while (l > 0 && *f != '\0')
